/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.author.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.author.model.Author;
import ru.ulstu.extractor.author.repository.AuthorRepository;

import java.util.Optional;

@Service
public class AuthorService {
    private final static Logger LOG = LoggerFactory.getLogger(AuthorService.class);
    private final AuthorRepository authorRepository;

    public AuthorService(AuthorRepository authorRepository) {
        this.authorRepository = authorRepository;
    }

    public Author findOrCreate(Author author) {
        Optional<Author> newAuthor = authorRepository.findByName(author.getName()).stream().findAny();
        if (newAuthor.isEmpty()) {
            LOG.debug("Author {} was saved.", author.getName());
            return authorRepository.save(author);
        }
        return newAuthor.get();
    }
}
