/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.branch.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.branch.repository.BranchRepository;
import ru.ulstu.extractor.commit.service.CommitService;
import ru.ulstu.extractor.core.BaseEntity;
import ru.ulstu.extractor.gitrepository.model.GitRepository;
import ru.ulstu.extractor.model.Commit;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class BranchService {
    private final static Logger LOG = LoggerFactory.getLogger(BranchService.class);
    private final BranchRepository branchRepository;
    private final CommitService commitService;

    public BranchService(BranchRepository branchRepository,
                         CommitService commitService) {
        this.branchRepository = branchRepository;
        this.commitService = commitService;
    }

    @Transactional
    public Branch save(Branch branch, List<Commit> commits) {
        LOG.debug("Start save {} branch with {} commits ", branch.getName(), commits.size());
        List<Integer> commitsToRemoveIds = branch.getCommits().stream().map(BaseEntity::getId).collect(Collectors.toList());
        branch.setCommits(commitService.save(commits));
        LOG.debug("Save branch {} ", branch.getName());
        branch = branchRepository.save(branch);
        LOG.debug("Clear {} commits ", commitsToRemoveIds.size());
        commitService.deleteWithEmptyIds();
        return branch;
    }

    public Branch addCommits(Branch branch, List<Commit> commits) {
        LOG.debug("Start add commits to {} branch with {} commits ", branch.getName(), commits.size());
        commitService.save(commits);
        LOG.debug("Save branch {} ", branch.getName());
        commitService.updateBranchId(commits, branch.getId());
        return branch;
    }

    public Branch findByRepositoryAndName(GitRepository gitRepository, String branchName) {
        return branchRepository.findByGitRepositoryAndName(gitRepository, branchName);
    }

    public List<Branch> findAll() {
        return branchRepository.findAll();
    }
}
