/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.commit.service;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.author.service.AuthorService;
import ru.ulstu.extractor.commit.repository.CommitRepository;
import ru.ulstu.extractor.model.Commit;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class CommitService {
    private final CommitRepository commitRepository;
    private final AuthorService authorService;
    private final JdbcTemplate jdbcTemplate;

    public CommitService(CommitRepository commitRepository,
                         AuthorService authorService,
                         JdbcTemplate jdbcTemplate) {
        this.commitRepository = commitRepository;
        this.authorService = authorService;
        this.jdbcTemplate = jdbcTemplate;
    }

    public void delete(List<Commit> commitsToRemove) {
        commitRepository.deleteAll(commitsToRemove);
    }

    public List<Commit> save(List<Commit> commits) {
        return commits.stream()
                .map(commit -> {
                    commit.setAuthor(authorService.findOrCreate(commit.getAuthor()));
                    return commitRepository.save(commit);
                }).collect(Collectors.toList());
    }

    public void deleteWithEmptyIds() {
        jdbcTemplate.update("DELETE FROM commit where branch_id is null");
    }

    public void updateBranchId(List<Commit> commits, Integer branchId) {
        List<String> commitIds = commits.stream().map(c -> c.getId().toString()).collect(Collectors.toList());
        String updateQuery = "update commit set branch_id = ? where id in (%s)";
        updateQuery = String.format(updateQuery, String.join(",", commitIds));
        jdbcTemplate.update(updateQuery, branchId);
    }


    public List<Commit> findByRepositoryIdAndName(Integer repositoryId, String branchName) {
        return commitRepository.findByRepositoryIdAndBranchName(repositoryId, branchName);
    }
}


