/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.gitrepository.controler;

import org.eclipse.jgit.api.errors.GitAPIException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.gitrepository.model.FilterForm;
import ru.ulstu.extractor.gitrepository.model.RepoForm;
import ru.ulstu.extractor.gitrepository.service.GitRepositoryService;
import ru.ulstu.extractor.gitrepository.service.IndexService;
import springfox.documentation.annotations.ApiIgnore;

import java.io.IOException;
import java.util.List;

import static ru.ulstu.extractor.core.Route.FILTER_COMMITS;
import static ru.ulstu.extractor.core.Route.INDEXING_NEW_REPOSITORY;

@Controller
@ApiIgnore
public class GitIndexingController {
    private final static Logger LOG = LoggerFactory.getLogger(GitIndexingController.class);
    private final GitRepositoryService gitRepositoryService;
    private final IndexService indexService;

    public GitIndexingController(GitRepositoryService gitRepositoryService,
                                 IndexService indexService) {
        this.gitRepositoryService = gitRepositoryService;
        this.indexService = indexService;
    }

    @GetMapping(INDEXING_NEW_REPOSITORY)
    public String indexNewRepo(Model model) {
        model.addAttribute(new RepoForm());
        return INDEXING_NEW_REPOSITORY;
    }

    @RequestMapping(value = INDEXING_NEW_REPOSITORY, method = RequestMethod.POST, params = "send")
    public String getBranch(@ModelAttribute RepoForm repoForm, Model model) {
        try {
            List<Branch> branches = gitRepositoryService.getRemoteBranches(repoForm.getRepo());
            model.addAttribute("branches", branches);
        } catch (Exception ex) {
            model.addAttribute("error", ex.getMessage());
        }
        return INDEXING_NEW_REPOSITORY;
    }

    @RequestMapping(value = INDEXING_NEW_REPOSITORY, method = RequestMethod.POST, params = "next")
    public String indexBranch(@ModelAttribute RepoForm repoForm, Model model, RedirectAttributes redirectAttributes) {
        model.addAttribute("filterForm", new FilterForm(repoForm.getRepo()));
        if (repoForm.getBranch() == null) {
            return INDEXING_NEW_REPOSITORY;
        } else {
            try {
                indexService.index(repoForm.getRepo(), repoForm.getBranch());
            } catch (IOException | GitAPIException ex) {
                ex.printStackTrace();
                model.addAttribute("error", ex.getMessage());
                return INDEXING_NEW_REPOSITORY;
            }
            redirectAttributes.addAttribute("repositoryUrl", repoForm.getRepo());
            redirectAttributes.addAttribute("branchName", repoForm.getBranch());
            return "redirect:/" + FILTER_COMMITS;
        }
    }
}
