/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.gitrepository.controler;

import org.eclipse.jgit.api.errors.GitAPIException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import ru.ulstu.extractor.gitrepository.service.GitRepositoryService;
import ru.ulstu.extractor.gitrepository.service.IndexService;
import ru.ulstu.extractor.model.Commit;
import ru.ulstu.extractor.util.HttpUtils;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.List;

import static ru.ulstu.extractor.gitrepository.controler.RepoController.URL;

@RestController
@RequestMapping(URL)
public class RepoController {
    private final static Logger LOG = LoggerFactory.getLogger(RepoController.class);
    public static final String URL = "/";
    private final GitRepositoryService gitRepositoryService;
    private final IndexService indexService;

    public RepoController(GitRepositoryService gitRepositoryService,
                          IndexService indexService) {
        this.gitRepositoryService = gitRepositoryService;
        this.indexService = indexService;
    }

    @GetMapping("commits")
    public List<Commit> getCommits(@RequestParam("repositoryUrl") String repositoryUrl,
                                   @RequestParam("branchName") String branchName,
                                   int commitsFrom,
                                   int commitsTo,
                                   HttpServletRequest request) throws GitAPIException, IOException {
        LOG.debug("Get commits {} {}. User ip {}", repositoryUrl, branchName, HttpUtils.getUserIp(request));
        return gitRepositoryService.getCommits(repositoryUrl, branchName, commitsFrom, commitsTo, true);
    }

    @GetMapping("index")
    public Boolean indexRepository(@RequestParam("repositoryUrl") String repositoryUrl,
                                   @RequestParam("branchName") String branchName,
                                   HttpServletRequest request) throws GitAPIException, IOException {
        LOG.debug("Index {} {}. User ip {}", repositoryUrl, branchName, HttpUtils.getUserIp(request));
        indexService.index(repositoryUrl, branchName);
        return true;
    }
}
