/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.gitrepository.model;

import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import ru.ulstu.extractor.core.BaseEntity;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import java.util.ArrayList;
import java.util.List;

@Entity
public class FileChange extends BaseEntity {
    private String file;

    private Boolean removed;

    private Boolean added;

    private Boolean containsEntity;

    private Boolean containsBusinessLogic;

    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @JoinColumn(name = "file_change_id", unique = true)
    @Fetch(FetchMode.SUBSELECT)
    private List<LineChange> lineChanges = new ArrayList<>();

    public FileChange() {
    }

    public FileChange(String file) {
        this.file = file;
    }

    public String getFile() {
        return file;
    }

    public Boolean getRemoved() {
        return removed;
    }

    public Boolean getAdded() {
        return added;
    }

    public List<LineChange> getLineChanges() {
        return lineChanges;
    }

    public void setFile(String file) {
        this.file = file;
    }

    public void setRemoved(boolean removed) {
        this.removed = removed;
    }

    public void setAdded(boolean added) {
        this.added = added;
    }

    public void setLineChanges(List<LineChange> lineChanges) {
        this.lineChanges = lineChanges;
    }

    public Boolean isRemoved() {
        return removed;
    }

    public Boolean isAdded() {
        return added;
    }

    public Boolean isContainsEntity() {
        return containsEntity;
    }

    public void setContainsEntity(boolean containsEntity) {
        this.containsEntity = containsEntity;
    }

    public Boolean getContainsEntity() {
        return containsEntity;
    }

    public void setContainsEntity(Boolean containsEntity) {
        this.containsEntity = containsEntity;
    }

    public Boolean getContainsBusinessLogic() {
        return containsBusinessLogic;
    }

    public void setContainsBusinessLogic(Boolean containsBusinessLogic) {
        this.containsBusinessLogic = containsBusinessLogic;
    }
}
