/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.gitrepository.service;

import com.sun.istack.NotNull;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.author.repository.AuthorRepository;
import ru.ulstu.extractor.commit.repository.CommitRepository;
import ru.ulstu.extractor.gitrepository.repository.GitRepositoryRepository;
import ru.ulstu.extractor.model.Commit;

import java.util.List;
import java.util.Map;

@Service
public class FilteringService {
    private final AuthorRepository authorRepository;
    private final CommitRepository commitRepository;
    private final GitRepositoryRepository gitRepositoryRepository;

    public FilteringService(AuthorRepository authorRepository,
                            CommitRepository commitRepository,
                            GitRepositoryRepository gitRepositoryRepository) {
        this.authorRepository = authorRepository;
        this.commitRepository = commitRepository;
        this.gitRepositoryRepository = gitRepositoryRepository;
    }

    public List<String> getRepositoryAuthors(@NotNull String repositoryUrl,
                                             @NotNull String branchName) {
        return authorRepository.findByRepositoryAndBranch(
                gitRepositoryRepository.findByUrl(repositoryUrl),
                branchName
        );
    }

    public List<String> getRepositoryAuthors(Integer branchId) {
        return authorRepository.findByBranchId(branchId);
    }

    public Page<Commit> getCommits(@NotNull String repositoryUrl,
                                   @NotNull String branchName,
                                   String author,
                                   String filter,
                                   Boolean entity,
                                   Pageable pageable) {
        return commitRepository.findByRepositoryAndBranch(
                pageable,
                gitRepositoryRepository.findByUrl(repositoryUrl),
                branchName,
                author,
                filter,
                entity
        );
    }

    public Map<String, Boolean> getEntityPresent() {
        return Map.of("Cодержит сущность", true,
                "Не содержит сущность", false);
    }
}
