/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.heuristic.service;

import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Locale;

import static ru.ulstu.extractor.heuristic.service.DetectorService.LangDetectScrupulousness.LOW;

@Service
public class DetectorService {
    public enum LangDetectScrupulousness {LOW, HIGH}

    private final static String BASE_URL = "http://localhost:8080/lang-detector.html";

    public String getDetectedLanguage(String fileName, String code, LangDetectScrupulousness scrupulousness) {
        if (scrupulousness == LOW) {
            return DirectoryService.getFileExtension(fileName).orElse("");
        }
        return getDetectedLanguage(code);
    }

    private String getDetectedLanguage(String code) {
        String selectedLang = null;
        try (WebClient webClient = new WebClient()) {
            webClient.setJavaScriptTimeout(60 * 1000);
            webClient.getOptions().setThrowExceptionOnScriptError(false);
            final HtmlPage page = webClient.getPage(BASE_URL);
            DomElement codeElement = page.getElementById("input");
            codeElement.setTextContent(code);
            DomElement button = page.getElementById("btn");
            button.click();
            DomElement output = page.getElementById("output");
            String outputString = output.getTextContent();
            List<String> langsStrings = Arrays.asList(outputString.split("\n"));
            int max = 0;
            for (String langString : langsStrings) {
                int currentVal = Integer.parseInt(langString.split(":")[1].trim());
                if (currentVal > max) {
                    max = currentVal;
                    selectedLang = langString.split(":")[0].trim();
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return selectedLang.toLowerCase(Locale.ROOT);
    }
}

