package ru.ulstu.extractor.rule.service;

import org.springframework.stereotype.Service;
import ru.ulstu.extractor.rule.model.AddRuleForm;
import ru.ulstu.extractor.rule.model.DbRule;
import ru.ulstu.extractor.rule.repository.RuleRepository;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.service.TimeSeriesService;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class DbRuleService {
    private final RuleRepository ruleRepository;
    private final TimeSeriesService timeSeriesService;
    private final AntecedentValueService antecedentValueService;

    public DbRuleService(RuleRepository ruleRepository,
                         TimeSeriesService timeSeriesService,
                         AntecedentValueService antecedentValueService) {
        this.ruleRepository = ruleRepository;
        this.timeSeriesService = timeSeriesService;
        this.antecedentValueService = antecedentValueService;
    }

    public List<DbRule> getList() {
        return ruleRepository.findAll();
    }

    public void saveRule(AddRuleForm addRuleForm) {
        if (addRuleForm.getRuleId() != null) {
            DbRule rule = ruleRepository.getOne(addRuleForm.getRuleId());
            rule.setConsequent(addRuleForm.getConsequent());
            rule.setFirstAntecedent(TimeSeriesType.valueOf(addRuleForm.getFirstAntecedentId()));
            rule.setFirstAntecedentValue(antecedentValueService.getById(addRuleForm.getFirstAntecedentValueId()));
            rule.setSecondAntecedent(TimeSeriesType.valueOf(addRuleForm.getSecondAntecedentId()));
            rule.setSecondAntecedentValue(antecedentValueService.getById(addRuleForm.getSecondAntecedentValueId()));
            ruleRepository.save(rule);
        } else {
            ruleRepository.save(new DbRule(antecedentValueService.getById(addRuleForm.getFirstAntecedentValueId()),
                    TimeSeriesType.valueOf(addRuleForm.getFirstAntecedentId()),
                    antecedentValueService.getById(addRuleForm.getSecondAntecedentValueId()),
                    TimeSeriesType.valueOf(addRuleForm.getSecondAntecedentId()),
                    addRuleForm.getConsequent()));
        }
    }

    public DbRule findById(Integer id) {
        return ruleRepository.getOne(id);
    }

    public AddRuleForm getAddRuleFormOrDefault(Integer id) {
        return id != null
                ? new AddRuleForm(ruleRepository
                .findById(id)
                .orElseThrow(() -> new RuntimeException("Правило не найдено")))
                : new AddRuleForm();
    }

    public void deleteById(Integer id) {
        ruleRepository.deleteById(id);
    }

    public List<String> getConsequentList() {
        return ruleRepository.findAll().stream().map(DbRule::getConsequent).collect(Collectors.toList());
    }
}
