package ru.ulstu.extractor.ts.creator;

import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.service.TimeSeriesService;

import java.util.Comparator;
import java.util.List;

import static ru.ulstu.extractor.ts.util.TimeSeriesDateMapper.mapTimeSeriesToInterval;

public abstract class AbstractTimeSeriesCreator {

    public abstract TimeSeriesType getTimeSeriesType();

    /**
     * Извлечь список точек временных рядов
     *
     * @param repositoryId
     * @param branchName
     * @return
     */
    public abstract List<TimeSeries> getTimeSeries(Integer repositoryId, String branchName);

    public abstract TimeSeriesService getTimeSeriesService();

    /**
     * Сохранить извлеченные временные ряды
     *
     * @param repositoryId
     * @param branch
     */
    public void addTimeSeries(Integer repositoryId, Branch branch) {
        // извлеченные временных рядов
        List<TimeSeries> timeSeries = getTimeSeries(repositoryId, branch.getName());

        // сгруппированные по временным интервалам точки временных рядов
        timeSeries.forEach(ts -> ts.setValues(mapTimeSeriesToInterval(getTimeSeriesService().getTimeSeriesInterval(), ts.getValues())));
        getTimeSeriesService().save(sortTimeSeries(timeSeries), branch);
    }

    private List<TimeSeries> sortTimeSeries(List<TimeSeries> timeSeries) {
        for (TimeSeries ts : timeSeries) {
            ts.getValues().sort(Comparator.comparing(TimeSeriesValue::getDate));
        }
        return timeSeries;
    }
}
