package ru.ulstu.extractor.ts.util;

import org.apache.commons.lang3.time.DateUtils;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Класс для регулировки дискретности временного ряда.
 * Можно создать временной ряд с точками с секундными отметками, затем суммировать значения,
 * применив одно из значений TimeSeriesInterval
 */
public class TimeSeriesDateMapper {

    public static List<TimeSeriesValue> mapTimeSeriesToInterval(TimeSeriesInterval timeSeriesInterval, List<TimeSeriesValue> timeSeriesValues) {
        List<TimeSeriesValue> trimmedTimeSeriesValues = timeSeriesValues
                .stream()
                .map(timeSeriesValue -> new TimeSeriesValue(trimTo(timeSeriesInterval, timeSeriesValue.getDate()),
                        timeSeriesValue.getValue()))
                .collect(Collectors.toList());
        Map<Date, Double> groupedTimeSeriesValues = trimmedTimeSeriesValues
                .stream()
                .collect(Collectors.groupingBy(TimeSeriesValue::getDate,
                        Collectors.summingDouble(TimeSeriesValue::getValue)));

        return groupedTimeSeriesValues.entrySet()
                .stream()
                .map(e -> new TimeSeriesValue(e.getKey(), e.getValue()))
                .collect(Collectors.toList());
    }

    public static TimeSeries mapTimeSeriesToInterval(TimeSeriesInterval timeSeriesInterval, TimeSeries timeSeries) {
        timeSeries.setValues(mapTimeSeriesToInterval(timeSeriesInterval, timeSeries.getValues()));
        return timeSeries;
    }

    private static Date trimTo(TimeSeriesInterval timeSeriesInterval, Date date) {
        return DateUtils.truncate(date, timeSeriesInterval.calendarField);
    }

    /**
     * Интервальность временного ряда при преобразовании
     */
    public enum TimeSeriesInterval {
        SECOND(Calendar.SECOND),
        MINUTE(Calendar.MINUTE),
        HOUR(Calendar.HOUR_OF_DAY),
        DAY(Calendar.DAY_OF_MONTH),
        WEEK(Calendar.WEEK_OF_MONTH),
        MONTH(Calendar.MONTH),
        YEAR(Calendar.YEAR);

        private final int calendarField;

        TimeSeriesInterval(int calendarField) {
            this.calendarField = calendarField;
        }
    }
}
