package ru.ulstu;

import org.junit.Assert;
import org.junit.Test;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.util.TimeSeriesDateMapper;

import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;

public class TimeSeriesMapperTest {
    @Test
    public void testMappingByDay() {
        Calendar c1 = GregorianCalendar.getInstance();
        c1.set(2020, 5, 1, 1, 1, 1);
        Calendar c2 = GregorianCalendar.getInstance();
        c2.set(2020, 5, 2, 2, 1, 1);
        TimeSeries timeSeries = new TimeSeries("Тестовый",
                Arrays.asList(new TimeSeriesValue(c1.getTime(), 10.0),
                        new TimeSeriesValue(c2.getTime(), 10.0)));
        TimeSeriesDateMapper mapper = new TimeSeriesDateMapper();
        timeSeries = mapper.mapTimeSeriesToInterval(TimeSeriesDateMapper.TimeSeriesInterval.MONTH, timeSeries);
        Assert.assertEquals(1, timeSeries.getValues().size());
        Assert.assertEquals(Integer.valueOf(20), timeSeries.getValues().get(0).getValue());
    }

    @Test
    public void testMappingByDayDifferent() {
        Calendar c1 = GregorianCalendar.getInstance();
        c1.set(2020, 5, 1, 1, 1, 1);
        Calendar c2 = GregorianCalendar.getInstance();
        c2.set(2020, 5, 2, 1, 1, 1);
        TimeSeries timeSeries = new TimeSeries("Тестовый",
                Arrays.asList(new TimeSeriesValue(c1.getTime(), 10.0),
                        new TimeSeriesValue(c2.getTime(), 10.0)));
        TimeSeriesDateMapper mapper = new TimeSeriesDateMapper();
        timeSeries = mapper.mapTimeSeriesToInterval(TimeSeriesDateMapper.TimeSeriesInterval.MONTH, timeSeries);
        Assert.assertEquals(1, timeSeries.getValues().size());
        Assert.assertEquals(Integer.valueOf(20), timeSeries.getValues().get(0).getValue());
    }

    @Test
    public void testMappingByMonth() {
        Calendar c1 = GregorianCalendar.getInstance();
        c1.set(2020, 5, 1, 1, 1, 1);
        Calendar c2 = GregorianCalendar.getInstance();
        c2.set(2020, 5, 2, 1, 1, 1);
        TimeSeries timeSeries = new TimeSeries("Тестовый",
                Arrays.asList(new TimeSeriesValue(c1.getTime(), 10.0),
                        new TimeSeriesValue(c2.getTime(), 10.0)));
        TimeSeriesDateMapper mapper = new TimeSeriesDateMapper();
        timeSeries = mapper.mapTimeSeriesToInterval(TimeSeriesDateMapper.TimeSeriesInterval.MONTH, timeSeries);
        Assert.assertEquals(1, timeSeries.getValues().size());
        Assert.assertEquals(Integer.valueOf(20), timeSeries.getValues().get(0).getValue());
    }

    @Test
    public void testMappingByMonthDifferent() {
        Calendar c1 = GregorianCalendar.getInstance();
        c1.set(2020, 5, 1, 1, 1, 1);
        Calendar c2 = GregorianCalendar.getInstance();
        c2.set(2020, 6, 2, 1, 1, 1);
        TimeSeries timeSeries = new TimeSeries("Тестовый",
                Arrays.asList(new TimeSeriesValue(c1.getTime(), 10.0),
                        new TimeSeriesValue(c2.getTime(), 10.0)));
        TimeSeriesDateMapper mapper = new TimeSeriesDateMapper();
        timeSeries = mapper.mapTimeSeriesToInterval(TimeSeriesDateMapper.TimeSeriesInterval.MONTH, timeSeries);
        Assert.assertEquals(2, timeSeries.getValues().size());
        Assert.assertEquals(Integer.valueOf(10), timeSeries.getValues().get(0).getValue());
    }

    @Test
    public void testMappingByYear() {
        Calendar c1 = GregorianCalendar.getInstance();
        c1.set(2020, 5, 1, 1, 1, 1);
        Calendar c2 = GregorianCalendar.getInstance();
        c2.set(2020, 5, 2, 1, 1, 1);
        TimeSeries timeSeries = new TimeSeries("Тестовый",
                Arrays.asList(new TimeSeriesValue(c1.getTime(), 10.0),
                        new TimeSeriesValue(c2.getTime(), 10.0)));
        TimeSeriesDateMapper mapper = new TimeSeriesDateMapper();
        timeSeries = mapper.mapTimeSeriesToInterval(TimeSeriesDateMapper.TimeSeriesInterval.YEAR, timeSeries);
        Assert.assertEquals(1, timeSeries.getValues().size());
        Assert.assertEquals(Integer.valueOf(20), timeSeries.getValues().get(0).getValue());
    }

    @Test
    public void testMappingByYearDifferent() {
        Calendar c1 = GregorianCalendar.getInstance();
        c1.set(2020, 5, 1, 1, 1, 1);
        Calendar c2 = GregorianCalendar.getInstance();
        c2.set(2021, 5, 2, 1, 1, 1);
        TimeSeries timeSeries = new TimeSeries("Тестовый",
                Arrays.asList(new TimeSeriesValue(c1.getTime(), 10.0),
                        new TimeSeriesValue(c2.getTime(), 10.0)));
        TimeSeriesDateMapper mapper = new TimeSeriesDateMapper();
        timeSeries = mapper.mapTimeSeriesToInterval(TimeSeriesDateMapper.TimeSeriesInterval.YEAR, timeSeries);
        Assert.assertEquals(2, timeSeries.getValues().size());
        Assert.assertEquals(Integer.valueOf(10), timeSeries.getValues().get(0).getValue());
    }
}
